/* ip_arp_rte_sup.ex.c */
/* Support procedures for the ARP example model */

/****************************************/
/*	Copyright (c) 1987 - 1994			*/
/*			by MIL 3, Inc.				*/
/*		(A Delaware Corporation)		*/
/*	3400 International Drive,  N.W.		*/
/*		Washington, D.C., U.S.A.		*/
/*			All Rights Reserved.		*/
/****************************************/

#include <opnet.h>
#include "ip_arp_sup.h"


arp_table*
arp_sup_table_setup (file_name)
	char			*file_name;
	{
	List*			line_list_ptr;
	arp_table*		at_ptr;


	/* Provides comprehensive routing table loading and parsing */
	/* services for ARP module. */
	FIN (arp_sup_table_setup (file_name))

	/* Load the list of text lines from the specified file. */
	/* Note: this procedure will quit the simulation if */
	/* file cannot be loaded, so it is assumed that there */
	/* are no problems upon returning. */
	line_list_ptr = arp_sup_table_load (file_name);

	/* Parse the contents of the obtained list into */
	/* an address map table */
	at_ptr = arp_sup_list_parse (line_list_ptr);

	/* In debug mode, if tracing is active, print the table */
	if (op_prg_odb_trace_active ())
		{
		arp_sup_table_print (at_ptr);
		}

	FRET (at_ptr);
	}


Compcode
arp_sup_phys_addr_get (at_ptr, node_num, phys_addr_ptr)
	arp_table*		at_ptr;
	int				node_num, *phys_addr_ptr;
	{
	int				i;


	/* Obtain a physical address from a node number, given an arp table. */
	FIN (arp_sup_phys_addr_get (at_ptr, node_num, phys_addr_ptr))

	/* Search through the table for a match */
	for (i = 0; i < at_ptr->num_mappings; i++)
		{
		if (at_ptr->mapping_set [i].node_num == node_num)
			{
			*phys_addr_ptr = at_ptr->mapping_set [i].phys_addr;
			FRET (OPC_COMPCODE_SUCCESS)
			}
		}


	/* Return failure status and invalid address. */
	*phys_addr_ptr = ARP_TBL_ADDR_NONE;
	FRET (OPC_COMPCODE_FAILURE)
	}


List*
arp_sup_table_load (file_name)
	char			*file_name;
	{
	List*			line_list_ptr;
	char			err_str [256];

	/* Read in an address-map table from an ascii */
	/* file adhering to format defined above. */
	FIN (arp_sup_table_load (file_name))

	/* Open and read the file into the list. */
	line_list_ptr = op_prg_gdf_read (file_name);

	/* Test for error in reading. */
	if (line_list_ptr == OPC_NIL)
		{
		sprintf (err_str, "File Name: %s", file_name);
		op_sim_end ("Package  : arp_sup", 
					"Error    : Unable to read address-map file", 
					err_str, "");	
		}

	/* Return the list of text lines. */
	FRET (line_list_ptr)
	}


arp_table*
arp_sup_list_parse (line_list_ptr)
	List*				line_list_ptr;
	{
	arp_table*			at_ptr;
	arp_trans*			trans_ptr;
	int					i, num_lines;
	char*				line;
	List				*temp_list_ptr, *field_list_ptr;


	/* Extract information from the lines of an ascii mapping table */
	/*  and construct a corresponding mapping table structure which */
	/* contains the translation information. */
	FIN (arp_sup_list_parse (line_list_ptr))

	/* Allocate a mapping table structure. */
	at_ptr = (arp_table*) op_prg_mem_alloc (sizeof (arp_table));

	/* Allocate a temporary table for holding lists. */
	temp_list_ptr = op_prg_list_create ();

	/* Scan through each of the lines, one at a time. */
	num_lines = op_prg_list_size (line_list_ptr);
	for (i = 0; i < num_lines; i++)
		{
		/* Obtain the i_th line. */
		line = op_prg_list_access (line_list_ptr, i);

		/* Decompose it into fields (field boundaries are */
		/* indicated by spaces, tabs,  slashes, or commas. */
		field_list_ptr = op_prg_str_decomp (line, " ,/\t");

		/* Format for a line is as follows: */
		/* <node number> <physical address>  */
		/* Incomplete lines are skipped. */
		if (op_prg_list_size (field_list_ptr) < 2)
			continue;

		/* Create an address translation structure. */
		trans_ptr = (arp_trans*) 
			op_prg_mem_alloc (sizeof (arp_trans));

		/* Transfer the parsed fields into the structure */
		trans_ptr->node_num = atoi (op_prg_list_access (field_list_ptr, ARP_TBL_NODE_NUM_FIELD));
		trans_ptr->phys_addr = atoi (op_prg_list_access (field_list_ptr, ARP_TBL_PHYS_ADDR_FIELD));

		/* Deallocate the memory for the field list and its contents. */
		op_prg_list_free (field_list_ptr);
		op_prg_mem_free (field_list_ptr);

		/* Append the address translation structure to the temporary list. */
		op_prg_list_insert (temp_list_ptr, trans_ptr, OPC_LISTPOS_TAIL);
		}

	/* If the translation list is empty (i.e., all instructions */
	/* were malformed), take care not to perform allocations. */
	if (op_prg_list_size (temp_list_ptr) == 0)
		{
		at_ptr->num_mappings = 0;
		at_ptr->mapping_set = OPC_NIL;
		}
	else{
		/* Dimension and allocate the dynamic array of translation entries */
		at_ptr->num_mappings = op_prg_list_size (temp_list_ptr);
		at_ptr->mapping_set = (arp_trans*) 
			op_prg_mem_alloc (at_ptr->num_mappings * sizeof (arp_trans));
	
		/* Transfer the contents of the temporary list into the array. */
		for (i = 0; i < at_ptr->num_mappings; i++)
			{
			op_prg_mem_copy (op_prg_list_access (temp_list_ptr, i), 
				&(at_ptr->mapping_set [i]), sizeof (arp_trans));
			}
		}


	/* Deallocate the temporary list contents and the list itself. */
	op_prg_list_free (temp_list_ptr);
	op_prg_mem_free (temp_list_ptr);

	/* Return the constructed table. */
	FRET (at_ptr);
	}


void
arp_sup_table_print (at_ptr)
	arp_table*			at_ptr;
	{
	int					i;

	/* Print the contents of an arp mapping table. */
	FIN (arp_sup_table_print (at_ptr))

	if (!at_ptr->num_mappings)
		{
		printf (" Mapping table is empty\n");
		}

	else{
		for (i = 0; i < at_ptr->num_mappings; i++)
			{
			printf ("%d)	(%d) ->	(%d)\n", 
				i,
				at_ptr->mapping_set [i].node_num,
				at_ptr->mapping_set [i].phys_addr);
			}
		}

	printf ("--------------------------------------\n");

	FOUT
	}










